-- Nest Announcer for 25-man heroic
-- Created by BlueWarlord
-- Based on the work by xLevante, see http://www.curse.com/addons/wow/ji-kun-nest-announcer
-- License: LGPL 3.0

NestAnnouncer = LibStub("AceAddon-3.0"):NewAddon("NestAnnouncer", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")

--------------------------------- Global variables ---------------------------------

local nest
local nestName
local timerHandle
local wipingDaemonHandle -- Used to track the timer implementing the combat tracking daemon
local timeDelay = 30 -- Should be 30 for production

-- Use PLAYER_REGEN_DISABLED for testing, use INSTANCE_ENCOUNTER_ENGAGE_UNIT for the real deal
local COMBAT_START_EVENT = "INSTANCE_ENCOUNTER_ENGAGE_UNIT"
local COMBAT_END_EVENT = "PLAYER_REGEN_ENABLED"

local defaults = {
  profile = {
    GAT = "Tank A Platform",
    GAH = "Healer A Platform",
    GBT = "Tank B Platform",
    SPARE1 = "Spare DPS",
	FT = "Flying tank",
    rwToggle = true,
    sayToggle = false,
    yellToggle = false,
	whisperToggle = true,
  },
}

local options =
{
  handler = NestAnnouncer,
  type = "group",
  args =
  {
    header1 =
    {
      type = "header",
      name = "People staying on the platform",
      order = 0,
      width = "full",
    },
    inputGAT =
    {
      type = "select",
      name = "Tank A Platform",
	  values = "getRaidMembers",
      get = "getGAT",
      set = "setGAT",
      order = 1,
      width = "normal",
    },
	inputGBT =
    {
      type = "select",
      name = "Tank B Platform",
	  values = "getRaidMembers",
      get = "getGBT",
      set = "setGBT",
      order = 2,
      width = "normal",
    },
    inputGAH =
    {
      type = "select",
      name = "Healer A Platform",
	  values = "getRaidMembers",
      get = "getGAH",
      set = "setGAH",
      order = 3,
      width = "normal",
    },	
    inputSpare =
    {
      type = "select",
      name = "Spare DPS",
	  values = "getRaidMembers",
      get = "getSpare",
      set = "setSpare",
      order = 5,
      width = "normal",
    },
	
	header3 =
	{
      type = "header",
      name = "Flying tank - Who is going to tank the adds on the nests?",
      order = 6,
      width = "full",
    },
	inputFT =
    {
      type = "select",
      name = "Flying tank",
	  values = "getRaidMembers",
      get = "getFT",
      set = "setFT",
      order = 7,
      width = "normal",
    },	

    header4 =
    {
      type = "header",
      name = "Announcement Options - How do you want to announce to your raid?",
      order = 14,
      width = "full",
    },
    toggleRW =
    {
      type = "toggle",
      name = "Raid Warning",
      desc = "Announce in Raid Warning (Requires Raid Leader)",
      get = "getRWToggle",
      set = "setRWToggle",
      order = 15,
      width = "normal",
    },
    toggleSay =
    {
      type = "toggle",
      name = "Say",
      desc = "Announce in Say",
      get = "getSayToggle",
      set = "setSayToggle",
      order = 16,
      width = "normal",
    },
    toggleYell =
    {
      type = "toggle",
      name = "Yell",
      desc = "Announce in Yell",
      get = "getYellToggle",
      set = "setYellToggle",
      order = 17,
      width = "normal",
    },
	toggleWhisper =
    {
      type = "toggle",
      name = "Whisper",
      desc = "Whisper each player",
      get = "getWhisperToggle",
      set = "setWhisperToggle",
      order = 18,
      width = "normal",
    },
  }
}

--------------------------------- Necessary Ace3 Functions ---------------------------------

function NestAnnouncer:OnInitialize()

  nest = 1

  self.db = LibStub("AceDB-3.0"):New("NestAnnouncerDB", defaults, "Default")
  LibStub("AceConfig-3.0"):RegisterOptionsTable("Nest Announcer", options)
  self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Nest Announcer", "Nest Announcer")

  self:RegisterChatCommand("na", "SlashCommand")

end

function NestAnnouncer:OnEnable()

end

function NestAnnouncer:OnDisable()
    -- Called when the addon is disabled
end

--------------------------------- Slash command menu ---------------------------------

function NestAnnouncer:HandleSlashCommand(arg1)

  if arg1 == "start" then
    NestAnnouncer:StartEncounter()
  elseif arg1 == "stop" then
    NestAnnouncer:StopEncounter()
  elseif arg1 == "test" then
	NestAnnouncer:TestEncounter()
  end

end

function NestAnnouncer:SlashCommand(arg1)

  if not arg1 or arg1:trim() == "" then
    InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
  else
    --LibStub("AceConfigCmd-3.0").HandleCommand(NestAnnouncer, "na", arg1)
    NestAnnouncer:HandleSlashCommand(arg1)
  end

end

--------------------------------- Other functions ---------------------------------

function NestAnnouncer:StartEncounter()

  NestAnnouncer:StopEncounter()
  -- Start the DBM pull timer
  SlashCmdList["DEADLYBOSSMODS"]("pull 10")
  -- Schedule the nest announcer to kick in when the pull timer is on 6
  timerHandle = NestAnnouncer:ScheduleTimer("AnnouncePlayers", 4)
  -- Create handler that waits on combat starts of the fight, 
  -- so we can wait till the boss no longer exists (i.e. we killed it, or it despawns)
  
  
  NestAnnouncer:RegisterEvent(COMBAT_START_EVENT, "StartOfCombat")
  

end

function NestAnnouncer:StopEncounter()
  NestAnnouncer:Print("StopEncounter()")
  nest = 1
  NestAnnouncer:CancelTimer(timerHandle)
  -- Stop the wipe detection handler
  NestAnnouncer:CancelTimer(wipingDaemonHandle)
  -- Make sure to stop the DBM pull timer if it is running
  SlashCmdList["DEADLYBOSSMODS"]("pull 0")
  -- Make sure we are no longer listening to combat starts
  NestAnnouncer:UnregisterEvent(COMBAT_START_EVENT)
  -- Make sure we are no longer listening to combat ends
  NestAnnouncer:UnregisterEvent(COMBAT_END_EVENT)
  
end

function NestAnnouncer:TestEncounter()
	
	-- Print out assignments
	SendChatMessage("----------------------------------", "RAID")
	SendChatMessage("Ji-Kul Nest announcer assignments", "RAID")
	local i
	for i = 1,5 do
		local msg = ""
		msg = "G"..i
		if i == 1 then 
			msg = msg .. " ({star}, UPPER {square}): "
		elseif i == 2 then
			msg = msg .. " (LOWER {square}): "
		elseif i == 3 then
			msg = msg .. " (LOWER {triangle}): "
		elseif i == 4 then
			msg = msg .. " ({diamond}, UPPER {triangle}): "			
		elseif i == 5 then
			msg = msg .. " ({cross}): "
		end
		
		local members = NestAnnouncer:GetMembersOfGroup(i)
		local j
		for j = 1, #members do
			msg = msg .. members[j] ..", "	
			SendChatMessage("As you can see this you are awesome. You are assigned to G"..i, "WHISPER", nil, members[j])	
		end	
		SendChatMessage(msg, "RAID")
	end
	SendChatMessage("Platform tanks: ".. self.db.profile.GAT..", "..self.db.profile.GBT, "RAID")
	SendChatMessage("As you can see this you are awesome. You are assigned to be a platform tank", "WHISPER", nil, self.db.profile.GAT)
	SendChatMessage("As you can see this you are awesome. You are assigned to be a platform tank", "WHISPER", nil, self.db.profile.GBT)
	SendChatMessage("Flying tank: ".. self.db.profile.FT, "RAID")
	SendChatMessage("As you can see this you are awesome. You are assigned to be the flying tank", "WHISPER", nil, self.db.profile.FT)
	SendChatMessage("Platform healer: "..self.db.profile.GAH, "RAID")
	SendChatMessage("As you can see this you are awesome. You are assigned to be the platform healer", "WHISPER", nil, self.db.profile.GAH)
	SendChatMessage("Spare dps: "..self.db.profile.SPARE1, "RAID")
	SendChatMessage("As you can see this you are awesome. You are assigned to be the spare dps", "WHISPER", nil, self.db.profile.SPARE1)
	
	SendChatMessage("----------------------------------", "RAID")

	
end

-- Function being called when the combat starts, starts the wiping detection daemon
function NestAnnouncer:StartOfCombat(eventName, ...)
	NestAnnouncer:Print("Combat starting")
	-- Register for the event that gets triggered when out of combat
	NestAnnouncer:RegisterEvent(COMBAT_END_EVENT, "EndOfCombat")	
end

-- Function that is called when COMBAT_END_EVENT is triggered by the game
function NestAnnouncer:EndOfCombat()	
	NestAnnouncer:Print(COMBAT_END_EVENT .. " triggered, starting wipe detection..")
	-- Start check to ensure boss is really dead
	wipingDaemonHandle = NestAnnouncer:ScheduleTimer("CheckForWipe", 0)
end

function NestAnnouncer:CheckForWipe() 
	if (UnitExists("Boss1") or UnitExists("Boss2") or 
		UnitExists("Boss3") or UnitExists("Boss4")) then
		wipingDaemonHandle = NestAnnouncer:ScheduleTimer("CheckForWipe", 3)
	else
		NestAnnouncer:Print("Boss does no longer exist ...")
		NestAnnouncer:StopEncounter()
	end
end

-- Combines two groups together, e.g. to get the tank into the message
function NestAnnouncer:CombineGroups(groupA, groupB)
	result = {
		name = "",
		members = {},
	}
	result.name = groupA.name .. " + "..groupB.name
	for i,v in ipairs(groupA.members) do
		table.insert(result.members,v)
	end
	for i,v in ipairs(groupB.members) do
		table.insert(result.members,v)
	end
	return result
end

-- Return an array with all the names of the people in the raid group
-- Filters out the people who should remain static, or the flyingtank
function NestAnnouncer:GetMembersOfGroup(groupNumber)
	local result, i
	result = {}
	for i = 1, GetNumGroupMembers() do
		local name, rank,  subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
		if not (name == nil) and not (online == nil) and (subgroup == groupNumber) and (isDead == nil) then
			-- Check if the person should be filtered out
			if not ((name == self.db.profile.GAT) or 
				(name == self.db.profile.GBT) or 
				(name == self.db.profile.GAH) or 
				(name == self.db.profile.SPARE1) or
				(name == self.db.profile.FT)) then
				table.insert(result, name)
			end
		end
	end
	if #result < 4 then
		-- We do not have enough people, use the spare!
		table.insert(result, self.db.profile.SPARE1)
	end
	return result
end

function NestAnnouncer:AnnouncePlayers()

  
  local group1, group2, group3, group4, group5, flyingTank

   
  -- Create assignments for the various groups and roles
  -- Group1 == GroupA
  group1 = {
	name = "G1",
	members = {},
  }  
  group1.members = NestAnnouncer:GetMembersOfGroup(1)
  
  group2 = {
	name = "G2",
	members = {}
  }
  group2.members = NestAnnouncer:GetMembersOfGroup(2)
	
  group3 = {
	name = "G3",
	members = {}
  }
  group3.members = NestAnnouncer:GetMembersOfGroup(3)
  
  -- Group4 == GroupB
  group4 = {
	name = "G4",
	members = {},
	--members = {self.db.profile.GBH,self.db.profile.GBD1,
		--self.db.profile.GBD2,self.db.profile.GBD3}  
  }
  group4.members = NestAnnouncer:GetMembersOfGroup(4)
  
  group5 = {
	name = "G5",
	members = {}
  }
  group5.members = NestAnnouncer:GetMembersOfGroup(5)
  
  flyingTank = {
	name = self.db.profile.FT,
	members = {self.db.profile.FT},
  }

  if nest == 1 then
	-- 0 Seconds
    NestAnnouncer:AnnounceNest(nest,group1,"LOWER NEST {star}")

  elseif nest == 2 then
    -- 30 seconds
	NestAnnouncer:AnnounceNest(nest,group2,"LOWER NEST {square}")    
    NestAnnouncer:AnnounceNest(nest,flyingTank,"LOWER NEST {square}")    

  elseif nest == 3 then
	-- 60 seconds
    NestAnnouncer:AnnounceNest(nest,group3,"LOWER NEST {triangle}")    
	
  elseif nest == 4 then
    -- 90 seconds
    NestAnnouncer:AnnounceNest(nest,group1,"UPPER NEST {star}")
    NestAnnouncer:AnnounceNest(nest,group4,"LOWER {diamond}")

  elseif nest == 5 then
    -- 120 seconds
	NestAnnouncer:AnnounceNest(nest,group1,"UPPER NEST {square}")
    NestAnnouncer:AnnounceNest(nest,
		NestAnnouncer:CombineGroups(flyingTank,group5),"LOWER NEST {cross}")

  elseif nest == 6 then
	-- 150 seconds
    NestAnnouncer:AnnounceNest(nest,group4,"UPPER CENTER {diamond}")

  elseif nest == 7 then
	-- 180 seconds
    NestAnnouncer:AnnounceNest(nest,group1,"LOWER NEST {star}")
	NestAnnouncer:AnnounceNest(nest,group4,"UPPER NEST {triangle}")
	
  elseif nest == 8 then
	-- 210 seconds
	NestAnnouncer:AnnounceNest(nest,group2,"LOWER NEST {square}")    
	NestAnnouncer:AnnounceNest(nest,
		NestAnnouncer:CombineGroups(flyingTank,group5),"UPPER NEST {cross}")

  elseif nest == 9 then
	-- 240 seconds
	NestAnnouncer:AnnounceNest(nest,group3,"LOWER NEST {triangle}")    
  elseif nest == 10 then
	-- 270 seconds
	NestAnnouncer:AnnounceNest(nest,group1,"UPPER NEST {star}")
	NestAnnouncer:AnnounceNest(nest,group4,"LOWER NEST {diamond}")
	
  elseif nest == 11 then
	-- 300 seconds
    NestAnnouncer:AnnounceNest(nest,
		NestAnnouncer:CombineGroups(flyingTank,group1),"UPPER NEST {square}")
	NestAnnouncer:AnnounceNest(nest,group5,"LOWER NEST {cross}")
	
  elseif nest == 12 then
	-- 330 seconds
	NestAnnouncer:AnnounceNest(nest,group1,"LOWER NEST {star}")
	NestAnnouncer:AnnounceNest(nest,group4,"UPER NEST CENTER {diamond}")

  elseif nest == 13 then
	-- 360 seconds
	NestAnnouncer:AnnounceNest(nest,group2,"LOWER NEST {square}")
	NestAnnouncer:AnnounceNest(nest,group4,"UPPER NEST {triangle}")
    
  elseif nest == 14 then
	-- 390 seconds
	NestAnnouncer:AnnounceNest(nest,group5,"UPPER NEST {cross}")
	NestAnnouncer:AnnounceNest(nest,group1,"UPPER NEST {star}")
	NestAnnouncer:AnnounceNest(nest,
		NestAnnouncer:CombineGroups(flyingTank,group3),"LOWER NEST {triangle}")
	
	
  elseif nest == 15 then
	-- 420 seconds
    NestAnnouncer:AnnounceNest(nest,group1,"UPPER NEST {square}")
	NestAnnouncer:AnnounceNest(nest,group4,"LOWER NEST {diamond}")    
	
  elseif nest == 16 then
	-- 450 seconds
	NestAnnouncer:AnnounceNest(nest,group5,"LOWER NEST {cross}")
	NestAnnouncer:AnnounceNest(nest,group1,"LOWER NEST {star}")
	NestAnnouncer:AnnounceNest(nest,group4,"UPPER NEST CENTER {diamond}")    
	
  elseif nest == 17 then
	-- 480 seconds
	NestAnnouncer:AnnounceNest(nest,group2,"LOWER NEST {square}")
	NestAnnouncer:AnnounceNest(nest,
		NestAnnouncer:CombineGroups(flyingTank,group4),"UPPER NEST {triangle}")    
	
  elseif nest == 18 then
	-- 510 seconds
	NestAnnouncer:AnnounceNest(nest,group3,"LOWER NEST {triangle}")
	NestAnnouncer:AnnounceNest(nest,group5,"UPPER NEST {cross}")
    NestAnnouncer:AnnounceNest(nest,group1,"UPPER NEST {star}")
	
  elseif nest == 19 then
	-- 540 seconds
	NestAnnouncer:AnnounceNest(nest,group1,"UPPER NEST {square}")
	NestAnnouncer:AnnounceNest(nest,group4,"LOWER NEST {diamond}")
	
  elseif nest == 20 then
	-- 570 seconds
	NestAnnouncer:AnnounceNest(nest,group2,"LOWER NEST {square}")
	NestAnnouncer:AnnounceNest(nest,group5,"LOWER NEST {cross}")
	NestAnnouncer:AnnounceNest(nest,group4,"UPPER NEST CENTER {diamond}")    	
	
  else
	NestAnnouncer:AnnounceNest(nil,nil,"No more nests!")
    NestAnnouncer:AnnounceNest(nil,nil,"Use remaining feathers and nuke!")

  end

  if nest <= 20 then
    nest = nest + 1
    timerHandle = NestAnnouncer:ScheduleTimer("AnnouncePlayers", timeDelay)
  else
    NestAnnouncer:StopEncounter()
  end

end

-- Function to handle all the nest annoucements and whispers
function NestAnnouncer:AnnounceNest(nestNumber, group, nestLocation)
	-- Determine what global channel to announce on
	local announceType, performAnnounce
	-- Iterate over the global announce types and broadcast if need be
	for i = 1, 3 do
		performAnnounce = false
		if i == 1 then
			announceType = "RAID_WARNING"
			if self.db.profile.rwToggle == true then performAnnounce = true end
		elseif i == 2 then
			announceType = "SAY"
			if self.db.profile.sayToggle == true then performAnnounce = true end
		else
			announceType = "YELL"
			if self.db.profile.yellToggle == true then performAnnounce = true end
		end
		
		if performAnnounce == true then
			-- We should indeed announce for global channel of announceType
			-- Check if we have a nest or just a message
			if (nestNumber == nil) or (group == nil) then
				SendChatMessage(nestLocation, announceType)
			else
				SendChatMessage(nestNumber..") "..group.name.." -> "..nestLocation, announceType)
			end
		end		
	end
	
	-- Handle the whispers to individual players
	if not (group == nil) and self.db.profile.whisperToggle then
		for i,member_name in ipairs(group.members) do
			SendChatMessage(nest..") --> ".. nestLocation, "WHISPER", nil, member_name)
		end
	end
end

function NestAnnouncer:getGAT(info)
    return self.db.profile.GAT
end

function NestAnnouncer:setGAT(info, value)
    self.db.profile.GAT = tostring(value)
end

function NestAnnouncer:getGAH(info)
    return self.db.profile.GAH
end

function NestAnnouncer:setGAH(info, value)
    self.db.profile.GAH = value
end

function NestAnnouncer:getGBT(info)
    return self.db.profile.GBT
end

function NestAnnouncer:setGBT(info, value)
    self.db.profile.GBT = value
end

function NestAnnouncer:getSpare(info)
    return self.db.profile.SPARE1
end

function NestAnnouncer:setSpare(info, value)
    self.db.profile.SPARE1 = value
end

function NestAnnouncer:getFT(info)
    return self.db.profile.FT
end

function NestAnnouncer:setFT(info, value)
    self.db.profile.FT = value
end

function NestAnnouncer:getRWToggle(info)
    return self.db.profile.rwToggle
end

function NestAnnouncer:setRWToggle(info, value)
    self.db.profile.rwToggle = value
end

function NestAnnouncer:getRaidMembers(info)
	local result,i
	result	= {}
	for i = 1, GetNumGroupMembers() do
		local name, rank,  subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i)
		if not (name == nil) then
			result[name] = name
		end
	end
	
    return result
end



function NestAnnouncer:getSayToggle(info)
    return self.db.profile.sayToggle
end

function NestAnnouncer:setSayToggle(info, value)
    self.db.profile.sayToggle = value
end



function NestAnnouncer:getYellToggle(info)
    return self.db.profile.yellToggle
end

function NestAnnouncer:setYellToggle(info, value)
    self.db.profile.yellToggle = value
end

function NestAnnouncer:getWhisperToggle(info)
    return self.db.profile.whisperToggle
end

function NestAnnouncer:setWhisperToggle(info, value)
    self.db.profile.whisperToggle = value
end